function init.game()
  -- If it's a new game...
  if not playing then
    playing = true
    ram = {
      weather = {
        t = 0,
        rain = 1,
        horizon = love.graphics.newCanvas(1, 2),
        colors = {
          {
            {255, 128, 192},  -- Top horizon color
            {255, 128, 128},  -- Bottom horizon color
            {255, 192, 192},  -- Scene color
          },
          {
            {0, 128, 192},
            {64, 192, 255},
            {255, 255, 255},
          },
          {
            {0, 64, 96},
            {128, 64, 128},
            {64, 128, 192},
          },
          {
            {0, 0, 0},
            {0, 0, 0},
            {64, 64, 64},
          },
        },
      },
      --total time passed
      timer = 0,
      --when new fish can spawn
      fishtimer = 0,
      --basically how much time you have left, you add to it when you catch fish.
      score = 0xFF,
      totalscore = 0,
      --lights up water at the cost of visibility
      flashlight = false,
      --rain effect strength between 0 and 1
      rain = 0,
      --how much flashlight time you have left
      flashlighttimer = 0xFF,
      --player boat
      boat = {
        x = 320,
        v = 0, -- velocity
      },
      --idling boat
      net = {
          x = 0,
      },
      hook = {
        x = 320,
        y = 240,
        vy = 0,
        --lastOnTop is used to tell when the hook hit the water, so it can splash
        lastOnTop = true,
        --fish ON the hook
        fish = {},
      },
      -- fish in water
      fish = {},
    }
    --sets up array for fish, 8 rows with up to 8 fish each
    for i = 1,8 do
      ram.fish[i] = {}
      for j = 1,8 do
        ram.fish[i][j] = {
          kind = 0,
          x = 0,
          right = false,
          --Random animation frame offset
          offset = 0,
        }
      end
    end
  end
  resize.game(love.graphics.getWidth(), love.graphics.getHeight())
  playSound("rain")
end

function update.game(dt)
  ram.timer = ram.timer + dt
  ram.weather.t = (ram.weather.t + dt/32)%4

  -- Update sound
  -- Boat water sloshing
  if math.abs(ram.boat.v) > 0 then
    sfx.surf:setVolume(math.abs(ram.boat.v/256))
    if not sfx.surf:isPlaying() then playSound("surf") end
    sfx.surf:setPosition(ram.boat.x/640 - .5, 0.1, 0.1)
  else
    love.audio.stop(sfx.surf)
  end

  -- Reeling
  if ram.hook.vy < 0 then
    sfx.reel:setVolume(-ram.hook.vy/16)
    sfx.reel:setPitch(-ram.hook.vy/10)
    playSound('reel')
    sfx.reel:setPosition(ram.hook.x/640 - .5, 0.1, 0.1)
  end

  -- Wind
  if ram.hook.y < 320 then
    sfx.wind:setVolume(math.abs(ram.hook.vy))
    sfx.wind:setPitch(.5 + math.abs(ram.hook.vy)/10 + math.abs(ram.hook.x-ram.boat.x)/128)
    playSoundLoop('wind')
    sfx.wind:setPosition(ram.hook.x/640 - .5, 0.1, 0.1)
  else
    sfx.wind:stop()
  end

  -- Rain
  ram.weather.rain = mouse.y/screen.h
  sfx.rain:setVolume(ram.weather.rain*0.5)

  -- Boat rain
  --[[
  playSoundLoop('boatrain')
  sfx.boatrain:setVolume(0.3 - math.abs(ram.boat.x-ram.net.x)/640)
  sfx.boatrain:setPosition((ram.net.x-ram.boat.x)/640, 0.1, 0.1)
  --]]

  -- Update particle systems
  particle.splashL:update(dt)
  particle.splashR:update(dt)
  particle.splashHook:update(dt)
  particle.dragHook:update(dt)

  -- Adds hook dragging particle effect
  if ram.hook.y > 460 then
    particle.dragHook:setEmissionRate(math.abs(math.ceil(ram.boat.x-ram.hook.x)+1))
  else
    particle.dragHook:setEmissionRate(0)
  end

  -- Updates when fish can spawn
  ram.fishtimer = ram.fishtimer + dt
  if ram.fishtimer >= 0.25 then
    spawnFish()
  end

  -- Cut down on time you have left
  ram.score = math.min(ram.score - dt*2, 255)

  -- Handle flashlight
  if ram.flashlight then
    ram.flashlighttimer = math.min(game.flashlighttimer - dt*2, 255)
  elseif ((ram.timer/32)%32) <=8 then
    ram.flashlighttimer = math.min(ram.flashlighttimer + dt*2, 255)
  end

  -- Moves idling boat, "net"
  ram.net.x = (math.cos(ram.timer * 0.25) * 640) / math.pi + 320

  -- Moves main boat
  if isDown('right') and ram.boat.x <= 640 then
    ram.boat.v = ram.boat.v + 1024*dt
  elseif isDown('left') and ram.boat.x >= 0 then
    ram.boat.v = ram.boat.v - 1024*dt
  else
    ram.boat.v = ram.boat.v - (ram.boat.v * dt * 2)
    if math.abs(ram.boat.v) < 4 then ram.boat.v = 0 end
  end
  ram.boat.v = math.clamp(ram.boat.v, -256, 256)
  ram.boat.x = ram.boat.x + dt * ram.boat.v
  ram.boat.x = math.clamp(ram.boat.x, -128, 640 + 128)

  -- Moves fish
  for i = 1,8 do
    for j = 1,8 do
      if ram.fish[i][j].type ~= 0 then
        if ram.fish[i][j].right then
          ram.fish[i][j].x = ram.fish[i][j].x + (dt*64) * (1 + ram.fish[i][j].kind/4)
        else
          ram.fish[i][j].x = ram.fish[i][j].x - (dt*64) * (1 + ram.fish[i][j].kind/4)
        end
      end
    end
  end

  -- despawn fish
  -- because fish start out at 0 or 1, and they move BEFORE
  -- this code, they can safely be despawned
  for i = 1,8 do
    for j = 1,8 do
      if ram.fish[i][j].x <= 0 or ram.fish[i][j].x >= 640 then
        ram.fish[i][j].kind = 0
      end
    end
  end

  --moves hook
  --if the hook's underwater...
  if ram.hook.y >= 320 then
    --"and hook.vy >=4" makes sure you can't stop a pull short and cheat
    if isDown('up') and ram.hook.vy >= -2 then ram.hook.vy = -2 end
    if isDown('pull') then ram.hook.vy = -12 end
    ram.hook.vy = math.clamp(ram.hook.vy, -12, 2)
    if isDown('down') and ram.hook.vy >= -4 then ram.hook.vy = 4 end
    if not ram.hook.lastOnTop then
      ram.hook.lastOnTop = not ram.hook.lastOnTop
      particle.splashHook:setEmissionRate(50)
      playSound('splash')
    else
      particle.splashHook:setEmissionRate(0)
    end
  else
    if ram.hook.lastOnTop then
      ram.hook.lastOnTop = not ram.hook.lastOnTop
      particle.splashHook:setEmissionRate(50)

      playSound('up')
    else
      particle.splashHook:setEmissionRate(0)
    end
  end
  -- tries to put the hook at ram.boat.x - 64 (the rear of the boat) instead of the middle to simulate drag
  ram.hook.x = ram.hook.x + (ram.boat.x - ram.hook.x) * dt * 2
  ram.hook.vy = ram.hook.vy + dt * 8
  -- Adds velocity while clamping its coordinates
  ram.hook.y = math.clamp(ram.hook.y + ram.hook.vy * dt * 32, 0, 480 - 16)


  -- attatches fish to the line
  -- gets which row the hook is on
  local tmp = math.ceil((ram.hook.y - 304)/16)
  if ram.hook.vy < -4 and tmp >= 1 and tmp <= 8 then
    for i = 1,8 do
      if ram.fish[i][tmp].x >= ram.hook.x - 24 - (ram.boat.x-ram.hook.x + 24)/4 and ram.fish[i][tmp].x <= ram.hook.x + 24 - (ram.boat.x-ram.hook.x + 24)/4 and ram.fish[i][tmp].kind ~= 0 then
        ram.hook.fish[table.getn(ram.hook.fish) + 1] = {
          kind = ram.fish[i][tmp].kind,
          --the fishs' animation offset also determines its' random size
          offset = ram.fish[i][tmp].offset,
          right = ram.fish[i][tmp].right,
        }
        ram.fish[i][tmp].kind = 0
        playSound('cut')
      end
    end
  end

  --puts fish in the boat
  if ram.hook.vy > 0 and ram.hook.y >= 320 then
    if ram.hook.x >= ram.net.x - 64 and ram.hook.x <= ram.net.x + 64 then
      for i = 1, table.getn(ram.hook.fish) do
        ram.score = math.clamp(ram.score + ram.hook.fish[i].kind * 16, 0, 255)
        ram.totalscore = ram.totalscore + ram.hook.fish[i].kind * 16
        playSound('slap')
      end
    end
    ram.hook.fish = {}
  end
end

function draw.game()
  -- seperated into its own function to make stereoscopy easier later on
  drawFrame()
  love.graphics.print(ram.totalscore, 0, 16)
end

function resize.game(w, h)
  shader.reload()
end

function drawFrame()

  --Start simulated 640*480 graphics
  love.graphics.push()
  canvas.main1:clear()
  love.graphics.setCanvas(canvas.main1)
  love.graphics.scale(screen.scale)

  -- Draw sky
  -- Draw horizon
  love.graphics.setCanvas(ram.weather.horizon)
  love.graphics.setColor(unpack(mix(1, ram.weather.colors, ram.weather.t)))
  love.graphics.point(0.5, 0.5)
  love.graphics.setColor(unpack(mix(2, ram.weather.colors, ram.weather.t)))
  love.graphics.point(0.5, 1.5)
  love.graphics.setColor(255,255,255,255)
  love.graphics.setCanvas(canvas.main1)
  love.graphics.draw(ram.weather.horizon, 0, 0, 0, screen.w, screen.h - 320 + screen.offset.y)

  --[[
  for i = 0, math.ceil(screen.w/512) do
    for j = 0, math.ceil(screen.h/512) do
      love.graphics.draw(tex.sky.storm, (i-1)*512 + (ram.timer*8)%512, j*512)
    end
  end
  --]]

  -- Draw sand
  shader.effect.sand:send('t', (512/screen.w)*ram.timer )
  love.graphics.setShader(shader.effect.sand)
  love.graphics.draw(tex.sand, 0, screen.h-128, 0, screen.w/128, 1)
  love.graphics.setShader()

  -- Draw sand from hook dragging
  love.graphics.draw(particle.dragHook, ram.hook.x + screen.offset.x, screen.h)

  -- Draw fog in front of sand
  love.graphics.setColor(0, 32, 16, 255)
  love.graphics.rectangle('fill', 0, screen.offset.y + 320, screen.w, 52)
  love.graphics.draw(tex.gradient, 0, screen.h, 0, screen.w, -64)
  love.graphics.setColor(255, 255, 255, 255)

  -- Draw fish to canvas
  canvas.main2:clear()
  love.graphics.setCanvas(canvas.main2)
  for i = 1,8 do
    for j = 1,8 do
      if ram.fish[i][j].kind ~= 0 then
        if ram.fish[i][j].right then
          love.graphics.draw(
            tex.fish,
            quad.fish[math.clamp(math.ceil(((ram.timer*8 + ram.fish[i][j].offset)%8)), 1, 8)],
            ram.fish[i][j].x + screen.offset.x,
            j*12+336+screen.offset.y,
            0,
            1 - ram.fish[i][j].offset/16, 1 - ram.fish[i][j].offset/16,
            32,
            16
          )
        else
          love.graphics.draw(
            tex.fish,
            quad.fish[math.clamp(math.ceil(((ram.timer*8 + ram.fish[i][j].offset)%8)), 1, 8)],
            ram.fish[i][j].x + screen.offset.x,
            j*12+336+screen.offset.y,
            0,
            -1 + ram.fish[i][j].offset/16, 1 - ram.fish[i][j].offset/16,
            32,
            16
          )
        end
      end
    end
  end

  -- Draw alpha channel
  love.graphics.setCanvas(canvas.norm)
  love.graphics.setColor(0, 0, 0, 255)
  love.graphics.rectangle('fill', 0, 0, screen.w, screen.h)
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.rectangle('fill', screen.w-screen.offset.x, screen.offset.y + 320, screen.offset.x, 160 + screen.offset.y)
  love.graphics.draw(tex.fog, screen.offset.x - 32, screen.offset.y + 320, 0, 1, 160 + screen.offset.y)
  love.graphics.rectangle('fill', 0, screen.offset.y + 320, screen.offset.x, 160 + screen.offset.y)

  -- Draw fish canvas with transparency
  love.graphics.pop()
  love.graphics.setCanvas(canvas.main1)
  shader.effect.alpha:send('map', canvas.norm)
  love.graphics.setShader(shader.effect.alpha)
  love.graphics.draw(canvas.main2)
  love.graphics.setShader()
  love.graphics.push()
  love.graphics.scale(screen.scale)

  -- Draw floaters in water
  for i = 0, math.ceil(screen.w/256) do
    for j = 0, math.ceil((screen.h-160)/256) - 1 do
      love.graphics.draw(tex.float, i*256 - (ram.timer%8) * 32, screen.h-160 - screen.offset.y + j * 256)
      love.graphics.draw(tex.float, i*256 - ((ram.timer + math.sin(ram.timer)*0.125 + 1)%8) * 32, screen.h-160 - screen.offset.y + j * 256)
    end
  end

  -- Draw hook
  love.graphics.draw(tex.hook, ram.hook.x + screen.offset.x, ram.hook.y + screen.offset.y, (ram.boat.x-ram.hook.x + 24)/256, -math.sin(ram.boat.v/512) + .1, 0.5, 24, 8)
  -- Draw fish on the hook
  for i = 1, table.getn(ram.hook.fish) do
    if ram.hook.fish[i].right then
      love.graphics.draw(tex.fish, quad.fish[1], ram.hook.x + 32 - (ram.boat.x-ram.hook.x + 24)/8 + screen.offset.x, ram.hook.y + 16 + screen.offset.y, 0, 1 - ram.hook.fish[i].offset/16, 1 - ram.hook.fish[i].offset/16, 32, 0)
    else
      love.graphics.draw(tex.fish, quad.fish[1], ram.hook.x + 32 - (ram.boat.x-ram.hook.x + 24)/8 + screen.offset.x, ram.hook.y + 16 + screen.offset.y, 0, -1 + ram.hook.fish[i].offset/16, 1 - ram.hook.fish[i].offset/16, 0, 0)
    end
  end

  -- Draw 2nd boat
  love.graphics.setColor(128, 64, 64, 255)
  love.graphics.draw(tex.boat, screen.offset.x + ram.net.x, screen.offset.y + 320, 0, 1, 1, 64, 32)
  love.graphics.setColor(255, 255, 255, 255)
  -- Draw player boat
  love.graphics.draw(tex.boat, screen.offset.x + ram.boat.x, screen.offset.y + 320, 0, 1, 1, 64, 32)

  -- Draw underwater overlay
  love.graphics.setColor(0, 32, 16, 64)
  love.graphics.rectangle('fill', 0, screen.offset.y + 320, screen.offset.x * 2 + 640, screen.offset.y + 160)
  love.graphics.setColor(255, 255, 255, 255)

  -- Draw normal map for underwater wobble
  love.graphics.setCanvas(canvas.norm)
  for j = 0, math.ceil((screen.h-(320+screen.offset.y))/128) do
    for i = 0, math.ceil(screen.w/128) do
      love.graphics.draw(tex.norm.water, i * 128 - (ram.timer%4)*32, screen.offset.y + 320 + j*128, 0, 1, 1)
    end
  end

  -- Draws medium grey box, because 0.5 is center on the normal map shader
  love.graphics.setColor(128, 128, 128, 255)
  love.graphics.rectangle('fill', 0, 0, screen.w, screen.h - 160 + screen.offset.y)
  -- Draw normal map for rain
  if ram.weather.rain > 0 then
    love.graphics.setColor(255, 255, 255, ram.weather.rain*255)
    love.graphics.setScissor(0, 0, screen.rw, screen.rh - 160*screen.scale + screen.offset.y*screen.scale)
    for j = 0, math.ceil((screen.h-160)/128)+1 do
      for i = 0, screen.w/128 + 1 do
        love.graphics.draw(tex.norm.rain, i * 128, j*128 + (ram.timer%0.5)*256 - 256, 0, 0.25, 0.25)
      end
    end
    love.graphics.setScissor()
  end
  love.graphics.setColor(255, 255, 255, 255)

  -- Warp main screen for rain & underwater
  love.graphics.pop()
  canvas.main2:clear()
  love.graphics.setCanvas(canvas.main2)
  shader.effect.norm:send('map', canvas.norm)
  shader.effect.norm:send('fac', 0.05)
  love.graphics.setShader(shader.effect.norm)
  love.graphics.draw(canvas.main1)
  love.graphics.push()
  love.graphics.scale(screen.scale)
  love.graphics.setShader()

  -- Create normal map for reflection
  canvas.norm:clear()
  love.graphics.setCanvas(canvas.norm)
  for i = 0, screen.w/256 + 1 do
    love.graphics.draw(tex.norm.water, i * 256 - (ram.timer%4)*64, screen.offset.y + 288, 0, 2, 0.25)
  end
  love.graphics.setColor(255, 255, 255, 128)
  for i = 0, screen.w/256 + 1 do
    love.graphics.draw(tex.norm.water, i * 256 - (ram.timer%8)*32, screen.offset.y*2 + 288, 0, 2, 0.25)
  end
  love.graphics.setColor(255, 255, 255, 255)
  canvas.main1:clear()
  love.graphics.setCanvas(canvas.main1)

  -- Draw reflection
  love.graphics.pop()
  love.graphics.draw(canvas.main2)
  love.graphics.setScissor(0, screen.offset.ry + 320*screen.scale, screen.rw, screen.scale * 16)
  shader.effect.norm:send('fac', 0.5)
  shader.effect.norm:send('map', canvas.norm)
  love.graphics.setShader(shader.effect.norm)
  local offset = screen.offset.ry + screen.scale * 320
  love.graphics.draw(canvas.main2, 0, offset, 0, 1, -0.5, 0, offset)
  love.graphics.setShader()
  love.graphics.setScissor()
  love.graphics.push()
  love.graphics.scale(screen.scale)

  -- Draw fishing line
  love.graphics.setColor(128,128,128, 255)
  local x1 = ram.boat.x + screen.offset.x
  local y1 = 320 + math.clamp(ram.hook.vy*2, -64, 0) + screen.offset.y
  local x2 = ram.hook.x + screen.offset.x
  local y2 = ram.hook.y + screen.offset.y
  local tablelol = {}
  tablelol[1] = x1
  tablelol[2] = y1
  tablelol[15] = x2
  tablelol[16] = y2
  for i = 3,14,2 do
    tablelol[i]   = x1 - (x1-x2) * i/16
    tablelol[i+1] = y1*(1-i/16)+y2*(i/16)^.5
  end
  love.graphics.line(tablelol)
  love.graphics.setColor(255,255,255)

  -- Particle systems
  if ram.boat.v > 0 then
    particle.splashL:setEmissionRate(ram.boat.v*0.125)
    particle.splashL:setSpeed(ram.boat.v/3)
  else
    particle.splashL:setEmissionRate(0)
  end
  if ram.boat.v < 0 then
    particle.splashR:setEmissionRate(math.abs(ram.boat.v*0.125))
    particle.splashR:setSpeed(math.abs(ram.boat.v)/3)
  else
    particle.splashR:setEmissionRate(0)
  end

  canvas.norm:clear()
  love.graphics.setCanvas(canvas.norm)

  -- Draw particle systems to normal map
  love.graphics.draw(particle.splashL, screen.offset.x + ram.boat.x - 48, screen.offset.y + 320)
  love.graphics.draw(particle.splashR, screen.offset.x + ram.boat.x + 48, screen.offset.y + 320)
  love.graphics.draw(particle.splashHook, ram.hook.x + screen.offset.x, screen.offset.y + 320)

  canvas.main2:clear()
  love.graphics.setCanvas(canvas.main2)

  -- Render it
  shader.effect.norm:send('fac', 0.1)
  shader.effect.norm:send('map', canvas.norm)
  love.graphics.setShader(shader.effect.norm)
  love.graphics.pop()
  love.graphics.draw(canvas.main1, 0, 0, 0, 1, 1, 0, 0)
  love.graphics.push()
  love.graphics.scale(screen.scale)
  love.graphics.setShader()

  -- Render rain fog
  love.graphics.setColor(127, 127, 127, ram.weather.rain*64)
  love.graphics.rectangle('fill', 0, 0, screen.w+screen.offset.x*2, screen.offset.y + 336)
  love.graphics.setColor(255, 255, 255, 255)

  -- Do final draw
  love.graphics.pop()
  love.graphics.setCanvas()

  -- Set color depending on time
  local x, y, z = unpack(mix(3, ram.weather.colors, ram.weather.t))
  local val = (1 - ram.weather.rain*0x.4)
  love.graphics.setColor(x*val, y*val, z*val)
  love.graphics.draw(canvas.main2)
end

function spawnFish()
  ram.fishtimer = 0
  for i = 1,8 do
    -- Pick a random slot
    local fish = ram.fish[math.random(1,8)][math.random(1,8)]
    -- If it's empty...
    if fish.kind == 0 then
      if math.random(1, 2) == 1 then
        fish.kind = math.random(1,4)
        fish.right = true
        fish.x = 0
        fish.offset = math.random(1,7)
      else
        fish.kind = math.random(1,4)
        fish.right = false
        fish.x = 640
        fish.offset = math.random(1,7)
      end
      return
    end
  end
end