--require 'crash'
require 'maths'
require 'shader'
function love.load()
  -- Set to true if playing but paused
  playing = false
  require 'serial'
  -- Saves / restores settings
  if love.filesystem.exists('save.bin') then
    save = serial.unpack(love.filesystem.read('save.bin'))
  else
    resetsave()
  end

  controller = {
    refresh = function()
      if love.joystick.getJoystickCount() > 0 then
        controller.count = love.joystick.getJoystickCount()
        controller.object = love.joystick.getJoysticks()
        controller.lastheld = {}
        controller.held = {}
        controller.deadzone = 0.8
        if not controller.object[save.stick] then
          save.stick = 1
        end
      else
        controller.count = 0
      end
    end
  }
  controller.refresh()

  -- Loads all images in /tex/ recursively
  tex = {}
  local function ass(sys, dir)
    local tab = split(dir, '/')
    table.remove(tab, 1)
    if #tab > 0 then
      local j = tex
      for i, k in pairs(tab) do
        if not tex[tab[i]] then tex[tab[i]] = {} end
        j = tex[tab[i]]
      end
      tab = j
    else
      tab = tex
    end

    for i,k in pairs(sys) do
      if love.filesystem.isFile(dir .. k) then
        local i = string.sub(k, 1, -5)
        tab[i] = love.graphics.newImage(dir .. k)
      elseif love.filesystem.isDirectory(dir .. k) then
        local fs2 = love.filesystem.getDirectoryItems(dir .. k)
        ass(fs2, dir .. k .. '/')
      end
    end
  end
  local fs = love.filesystem.getDirectoryItems('/tex')
  ass(fs, 'tex/')

  -- Creates quads
  quad = {}
  quad.fish = {}
  for i=1,8 do
    --quad.fish[i] = love.graphics.newQuad((i-1)*64, 0, 64, 64, 512, 64)
    quad.fish[i] = love.graphics.newQuad((i-1)*32, 0, 32, 32, 256, 32)
  end

  --particle systems

  --to the left of player
  particle = {}
  particle.splashL = love.graphics.newParticleSystem(tex.splash,256)
  particle.splashL:setEmissionRate(0)
  particle.splashL:setParticleLifetime(.5,.2)
  particle.splashL:setSizeVariation(1)
  particle.splashL:setSpin(5)
  particle.splashL:setSpread(1)
  particle.splashL:setSpeed(150,150)
  particle.splashL:setDirection(-math.pi + .5)
  particle.splashL:start()
  particle.splashL:setLinearAcceleration(50,250)

  --to the right of player
  particle.splashR = love.graphics.newParticleSystem(tex.splash,256)
  particle.splashR:setEmissionRate(0)
  particle.splashR:setParticleLifetime(.5,.2)
  particle.splashR:setSizeVariation(1)
  particle.splashR:setSpin(5)
  particle.splashR:setSpread(1)
  particle.splashR:setSpeed(150,150)
  particle.splashR:setDirection(-.5)
  particle.splashR:start()
  particle.splashR:setLinearAcceleration(50,250)

  --hook splash
  particle.splashHook = love.graphics.newParticleSystem(tex.splash,256)
  particle.splashHook:setEmissionRate(0)
  particle.splashHook:setParticleLifetime(.5,.2)
  particle.splashHook:setSizeVariation(1)
  particle.splashHook:setSpin(5)
  particle.splashHook:setSpread(1)
  particle.splashHook:setSpeed(150,150)
  particle.splashHook:setDirection(-math.pi/2)
  particle.splashHook:start()
  particle.splashHook:setLinearAcceleration(50,250)

  --hook drag
  particle.dragHook = love.graphics.newParticleSystem(tex.drag,256)
  particle.dragHook:setEmissionRate(0)
  particle.dragHook:setParticleLifetime(.5,.2)
  particle.dragHook:setSizeVariation(1)
  particle.dragHook:setSpin(5)
  particle.dragHook:setSpread(1)
  particle.dragHook:setSpeed(150,150)
  particle.dragHook:setDirection(-math.pi)
  particle.dragHook:start()
  particle.dragHook:setLinearAcceleration(50,250)

  -- Sets font
  local font = love.graphics.newImageFont(tex.font, '!"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~ ')
  font:setFilter('nearest', 'nearest')
  love.graphics.setFont(font)

  -- Working memory for each 'virtual machine'
  ram = {}

  -- Current state
  state = 'game'

  -- The various functions a state can use
  init = {}
  draw = {}
  update = {}
  keypress = {}
  buttonpress = {}
  mousepress = {}
  mouserelease = {}
  resize = {}

  require 'intro'
  require 'menu'
  require 'game'
  require 'control'
  love.resize(love.graphics.getWidth(), love.graphics.getHeight())

  -- Load all music
  music = {}
  local fs = love.filesystem.getDirectoryItems('music')
  for i,k in pairs(fs) do
    music[string.sub(k, 1, -5)] = love.audio.newSource('music/' .. k, 'stream')
    music[string.sub(k, 1, -5)]:setLooping(true)
    music[string.sub(k, 1, -5)]:setVolume(0.2)
  end

  -- Load all SFX
  sfx = {}
  local fs = love.filesystem.getDirectoryItems('sfx')
  for i,k in pairs(fs) do
    if string.sub(k, -8, -5) == 'loop' then
      sfx[string.sub(k, 1, -10)] = love.audio.newSource('sfx/' .. k, 'static')
      sfx[string.sub(k, 1, -10)]:setVolume(0.1)
      sfx[string.sub(k, 1, -10)]:setLooping(true)
    else
      sfx[string.sub(k, 1, -5)] = love.audio.newSource('sfx/' .. k, 'static')
      sfx[string.sub(k, 1, -5)]:setVolume(0.1)
    end
  end
  love.resize(love.graphics.getWidth(), love.graphics.getHeight())
  -- Initializes game
  init[state]()
end

function love.update(dt)
  -- Update mouse position
  mouse = {
    x = love.mouse.getX()/screen.scale - screen.offset.x,
    y = love.mouse.getY()/screen.scale - screen.offset.y
  }
  -- Check for newly connected controllers, update accordingly
  if controller.count ~= love.joystick.getJoystickCount() then
    controller.refresh()
    setstate('menu', false)
    return
  end

  if update[state] then
      update[state](dt)
  end
end

function love.draw()
  if draw[state] then
    draw[state]()
  end
  -- Print FPS
  love.graphics.print(love.timer.getFPS(), 0, 0)
end

function setstate(newstate, newram)
  if newram ~= false then ram = newram end
  collectgarbage('collect')
  state = newstate
  init[state]()
end

function love.resize(w, h)
  -- rw and rh are the *real* dimensions, the others are scaled to simulate 640x480
  screen = {rw = w, rh = h}
  screen.scale = math.min(screen.rw/640, screen.rh/480)
  screen.w = screen.rw/screen.scale
  screen.h = screen.rh/screen.scale
  screen.offset = {
    rx = 0.5 * (screen.rw - screen.scale*640),
    ry = 0.5 * (screen.rh - screen.scale*480)
  }
  screen.offset.x = screen.offset.rx/screen.scale
  screen.offset.y = screen.offset.ry/screen.scale
  --Minimum screen size is 640x480
  if resize[state] then
    resize[state]()
  end
end

function savedata()
  love.filesystem.write('save.bin', serial.pack(save))
end

-- Sets save to defaults - upon game crashes, savedata is reset as a failsafe
function resetsave()
  save = {
    highscore = 0,
    music = 'disable',
    sfx = true,
    stick = 1,
    stereo = false,
    stereostr = 1.0,
    controller = {
      {name = 'up'    , key = 'up',     button = 'none'},
      {name = 'down'  , key = 'down',   button = 'none'},
      {name = 'left'  , key = 'left',   button = 'none'},
      {name = 'right' , key = 'right',  button = 'none'},
      {name = 'pull'  , key = ' ',      button = 'none'},
      {name = 'back'  , key = 'escape', button = 'none'},
    },
  }
  savedata()
end

function playMusic(song)
  if song ~= 'disable' then
    if not music[song]:isPlaying() then
      if save.music ~= 'disable' then
        music[save.music]:stop()
      end
      music[song]:play()
    end
  else
    if save.music ~= 'disable' then
      music[save.music]:stop()
    end
  end
end

function playSound(sound)
  if save.sfx then
    sfx[sound]:stop()
    sfx[sound]:play()
  end
end

function playSoundLoop(sound)
  if save.sfx then
    if not sfx[sound]:isPlaying() then
      sfx[sound]:play()
    end
  end
end

function split(str, pat)
  local t = {}
  local fpat = '(.-)' .. pat
  local last_end = 1
  local s, e, cap = str:find(fpat, 1)
  while s do
    if s ~= 1 or cap ~= '' then
      table.insert(t,cap)
    end
    last_end = e+1
    s, e, cap = str:find(fpat, last_end)
  end
  if last_end <= #str then
    cap = str:sub(last_end)
    table.insert(t, cap)
  end
  return t
end
--[[
function love.graphics.drawCanvas(canvas, x, y, r, sx, sy, ox, oy, kx, ky)
  love.graphics.setBlendMode('premultiplied')
  love.graphics.draw(canvas, x, y, r, sx, sy, ox, oy, kx, ky)
  love.graphics.setBlendMode('alpha')
end]]--