function init.menu()
  love.audio.stop()
  ram = {
    option = 'main',
    cursor = 1,
    main = {
      -- 1
      {
        name = 'play',
        func = function()
          setstate('game', false)
        end
      },
      -- 2
      {
        name = 'new game',
        func = function()
          playing = false
          setstate('game', nil)
          return
        end
      },
      -- 3
      {
        name = 'settings',
        func = function()
          ram.cursor = 1
          ram.option = 'settings'
        end
      },
      -- 4
      {
        name = 'controller',
        func = function()
          if controller.count > 0 then
            ram.cursor = 1
            ram.option = 'controller'
          end
        end
      },
      -- 5
      {
        name = 'exit',
        func = function()
          love.event.quit()
        end
      },
    },
    controller = {},
    settings = {
      --name: Text to display
      --savename: save[savename] will be set to options.value upon saving
      --options:
        --name: description to display
        --value: value to set save[savename]
        --selected: which one of the settings is currently selected
        --func: code to run upon being selected
      -- 1
      {
        name = 'bgm',
        savename = 'music',
        selected = 1,
        options = {
        },
      },
      -- 2
      {
        name = 'sfx',
        savename = 'sfx',
        selected = 1,
        options = {
          {
            name = 'enabled',
            value = true,
            func = function()
              save.sfx = true
              savedata()
            end
          },
          {
            name = 'disabled',
            value = false,
            func = function()
              save.sfx = false
              savedata()
            end
          },
        },
      },
      -- 3
      {
        name = 'joystick',
        savename = 'stick',
        selected = 1,
        options = {
        },
      },
    },
    wait = 0,
  }

  --Generate music config
  local j = 0
  for i, k in pairs(music) do
    if j == 0 then
      j = 1
      ram.settings[1].options[j] = {
        name = 'disable',
        value = 'disable',
        func = function()
          playMusic('disable')
          save.music = 'disable'
          savedata()
        end
      }
    end
    j = j + 1
    ram.settings[1].options[j] = {
      name = i,
      value = i,
      func = function()
        playMusic(i)
        save.music = i
        savedata()
      end
    }
  end

  if controller.count > 0 then
    --Generate selected joystick config
    local j = 0
    for i, k in pairs(controller.object) do
      j = j + 1
      ram.settings[3].options[j] = {
        name = string.sub(string.lower(controller.object[i]:getName()), 1, 8),
        value = i,
        func = function()
          save.stick = i
          savedata()
        end
      }
    end

    --Generate controller config
    for i, k in ipairs(save.controller) do
      ram.controller[i] = {
        name = save.controller[i].name,
        func = function()
          if controller.count > 0 then
            ram.wait = 5
          end
        end
      }
    end
  else
    ram.settings[3].options[1] = {
      name = 'none',
      value = 1,
      func = function()
      end
    }
  end
  playMusic(save.music)

  -- Figure out which settings should be selected
  for j, l in ipairs(ram.settings) do
    for i, k in pairs(ram.settings[j].options) do
      if k.value == save[l.savename] then
        ram.settings[j].selected = i
        break
      end
    end
  end
end

function update.menu(dt)
  if ram.wait > 0 then
    if ram.tmpkey then
      save.controller[ram.cursor].button = ram.tmpkey
      ram.tmpkey = nil
      ram.wait = 0
      savedata()
    else
      ram.wait = math.max(ram.wait - dt, 0)
    end
  end
end

function draw.menu()
  --Start 640x480 graphics
  love.graphics.push()
  love.graphics.translate(screen.offset.rx, screen.offset.ry)
  love.graphics.scale(screen.scale)
  -- Draw cursor
  love.graphics.setColor(0, 128, 192, 192)
  love.graphics.rectangle(
    'fill',
    160,
    160 + 32*ram.cursor,
    320,
    32
  )
  love.graphics.setColor(255, 255, 255, 255)

  if ram.option == 'settings' then
    for i, k in ipairs(ram.settings) do
      love.graphics.print(k.name, 168, i * 32 + 168, 0, 2)
      love.graphics.print(k.options[k.selected].name, 384, i * 32 + 160, 0, 2)
      love.graphics.draw(tex.arrow, 336, i * 32 + 160, 0, 2)
      love.graphics.draw(tex.arrow, 576, i * 32 + 160, 0, -2, 2)
    end
  elseif ram.option == 'controller' then
    if ram.wait > 0 then
      love.graphics.print('waiting for input... ' .. math.ceil(ram.wait), 96, 56)
    end
    for i, k in ipairs(ram.controller) do
      -- Names
      love.graphics.print(
        k.name,
        96,
        64+i*16
      )
      -- Mappings
      love.graphics.print(
        save.controller[i].button,
        168,
        64+i*16
      )
      -- 'Clear' buttons
      love.graphics.draw(tex.x, 212, 56 + i*16)
    end
  elseif ram.option == 'main' then
    for i, k in ipairs(ram.main) do
      love.graphics.print(
        k.name,
        168,
        168+i*32,
        0, 2
      )
    end
  end
  -- End 640x480 graphics
  love.graphics.pop()
  -- Back button
  love.graphics.draw(tex.x, 0, 0, 0, 2*screen.scale)
end

function keypress.menu(key)
  --If we aren't waiting for a button press for the controller configuration...
  if ram.wait == 0 then
    if key == 'up' then
      ram.cursor = math.max(ram.cursor - 1, 1)
    elseif key == 'down' then
      ram.cursor = math.min(ram.cursor + 1, #ram[ram.option])
    elseif key == ' ' then
      if ram.option ~= 'settings' then
        ram[ram.option][ram.cursor].func()
      end
    elseif key == 'right' then
      if ram.option == 'settings' then
        ram.settings[ram.cursor].selected = (ram.settings[ram.cursor].selected) % #ram.settings[ram.cursor].options + 1
        ram.settings[ram.cursor].options[ram.settings[ram.cursor].selected].func()
      end
    elseif key == 'left' then
      if ram.option == 'settings' then
        ram.settings[ram.cursor].selected = (ram.settings[ram.cursor].selected - 2) % #ram.settings[ram.cursor].options + 1
        ram.settings[ram.cursor].options[ram.settings[ram.cursor].selected].func()
      end
    elseif key == 'escape' then
      if ram.option ~= 'main' then
        ram.option = 'main'
        ram.cursor = 1
      else
        if playing then
          setstate('game', false)
          return
        else
          love.event.quit()
        end
      end
    end
  end
end

function buttonpress.menu(button)
  if ram.wait > 0 then
    ram.tmpkey = button
  end
end

function mousepress.menu(x, y, button)
  if x > 0 and x < 32 * screen.scale and y > 0 and y < 32 * screen.scale then
    keypress.menu('escape')
  end
  if ram.wait == 0 then
    x = (x - screen.offset.x) / screen.scale
    y = (y - screen.offset.y) / screen.scale

    if ram.option == 'settings' then
      if x > 168 and x < 184 then
        for i, k in ipairs(ram.settings) do
          if
            y > i * 16 + 60
          and
            y < i * 16 + 76
          then
            ram.settings[i].selected = (ram.settings[i].selected - 2) % #ram.settings[i].options + 1
            ram.settings[i].options[ram.settings[i].selected].func()
          end
        end
      elseif x > 272 and x < 288 then
        for i, k in ipairs(ram.settings) do
          if
            y > i * 16 + 60
          and
            y < i * 16 + 76
          then
            ram.settings[i].selected = ram.settings[i].selected % #ram.settings[i].options + 1
            ram.settings[i].options[ram.settings[i].selected].func()
          end
        end
      end
    else
      if x > 94 and x < 166 then
        if y > 72 and y < #ram[ram.option] * 16 + 72 then
          ram.cursor = math.ceil((y-72)/16)
          ram[ram.option][ram.cursor].func()
          return
        end
      end
      -- Buttons to reset controls
      if ram.option == 'controller' then
        if x > 212 and x < 228 then
          if y > 72 and y < #ram[ram.option] * 16 + 72 then
            save.controller[math.ceil((y-72)/16)].button = 'none'
            savedata()
          end
        end
      end
    end
  end
end